#version 300 es
/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * apdShader.frag   :   measure APDs
 *
 * PROGRAMMER   :   ABOUZAR KABOUDIAN
 * DATE         :   Thu 06 Dec 2018 14:24:09 (EST)
 * PLACE        :   Chaos Lab @ GaTech, Atlanta, GA
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 */
precision highp float ;
precision highp int ;

in vec2 pixPos ;
uniform float       dt ;
uniform float       threshold ;

uniform sampler2D   voltage ;
uniform sampler2D   timeInfoIn ;
uniform sampler2D   apdInfoIn ;


layout (location = 0 ) out vec4 timeInfoOut ;
layout (location = 1 ) out vec4 apdInfoOut ;


void main(){
    vec4    apdInfo     = texture(apdInfoIn,  pixPos ) ;
    vec4    timeInfo    = texture(timeInfoIn, pixPos ) ;
    float   v           = texture(voltage   , pixPos ).r ;

    float   currentTime = timeInfo.r ;
    float   apStartTime = timeInfo.g ;
    float   started     = timeInfo.b ;
    float   apEndTime   = timeInfo.a ;
    float   apd         = apdInfo.r ;   /* AP Duration          */
    float   bcl         = apdInfo.g ;   /* Basic Cycle Length   */
    float   di          = apdInfo.b ;   /* Diastolic Interval   */
    float   lastApStartTime = apdInfo.a ;


    if (started > 0.5 ){
        if ( v < threshold ){
            started     = 0. ;
            apd         = currentTime - apStartTime ;

            bcl         = apStartTime - lastApStartTime ;
            di          = apStartTime - apEndTime ;
            apEndTime   = currentTime ;
            lastApStartTime = apStartTime ;
        }
    }else{
        if ( v> threshold ){
            started     = 1.0 ;
            apStartTime = currentTime ;
        }
    }

    currentTime += dt ;   /* march time           */

    timeInfoOut = vec4(currentTime, apStartTime,started,apEndTime) ;
    apdInfoOut  = vec4(apd, bcl, di,lastApStartTime ) ; 
     
    if (currentTime > 20000.){ 
        timeInfoOut = vec4(0.) ;
        apdInfoOut = vec4(0.) ;
    }
}
